unit DFWAV;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, MMSystem, ExtCtrls;

type
    tFlags = (SYNC, ASYNC);

const
    SND_SYNC: word = $0000;                {  play synchronously (default)}
    SND_ASYNC: Word = $0001;               {  play asynchronously }
    SND_NODEFAULT = $0002;                 {  don't use default sound }
    SND_MEMORY = $0004;                    {  lpszSoundName points to a memory file }
    SND_LOOP = $0008;                      {  loop the sound until next sndPlaySound }
    SND_NOSTOP = $0010;                    {  don't stop any currently playing sound }

    wFlags : array[tFlags] of Word = ( $0000, $00001);


type
  {TDFWAV = class(TComponent)}
  TDFWAV = class(TGraphicControl)
  private
    { Private-Deklarationen }
    PlaySound : function (lpszSoundName: PChar; uFlags: Word): Bool;
    MyFlag : tFlags;
    MyFileName : String;
    MyPlay : Boolean;
    procedure SetMyFlag(NewFlag: tFlags);
    procedure SetMyFileName(NewFileName: String);
    procedure PlayFileName(NewAction: Boolean);
    procedure DrawImage (var Message: TWMPaint); message wm_paint;
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
  published
    { Published-Deklarationen }
    constructor Create(AOwner: TComponent); override; { Create umleiten }
    destructor Destroy; override;                     { Destructor umleiten }
    Property Flags : tFlags read MyFlag write SetMyFlag;
    Property FileName : String read MyFileName write SetMyFileName;
    Property Play : Boolean read MyPlay write PlayFileName;
    property ShowHint;
    Property Visible;
    { Published-Proceduren }
    procedure Beep;
    procedure PlayWAV(FileName: String; Flags: Word);
    Procedure WaitWAV;
  end;

procedure Register;

implementation

constructor TDFWAV.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Height := 25;
  Width := 25;
end; {Create}

destructor TDFWAV.Destroy;
begin


     inherited Destroy;                          { originaler Destructor }
end; {Destroy}

procedure TDFWAV.SetMyFlag(NewFlag: tFlags);
begin
     MyFlag := NewFlag;
end; {SetMyFlag}

procedure TDFWAV.SetMyFileName(NewFileName: String);
begin
  MyFileName := NewFileName;
  Refresh;
end; {SetMyFileName}

procedure TDFWAV.DrawImage(var Message: TWMPaint);
var
  F: File of Byte;
  FLenght: LongInt;
  B: Byte;
  XPos: Integer;
  YPos,FPos: LongInt;
  Step: Integer;
  YMitte : Integer;
begin

  if Visible = False Then Exit;

  Canvas.Brush.Color := clBlack;

  Canvas.FillRect(Rect(0, 0, Width, Height));


  if not FileExists(FileName) then exit;

  try
    Canvas.TextOut(0,0,FileName);
    AssignFile(F, FileName);
    FileMode := 0;                               { ReadOnly }
    Reset(F);                                    { Datei ffnen }
    FLenght := FileSize(F);
    Step := FLenght div Width;
    XPos := 0;
    FPos := 45;
    YMitte := Height div 2;
    Canvas.Pen.Color := clRed;
    Canvas.MoveTo(0, YMitte);
    Canvas.LineTo(Width, YMitte);

    Canvas.Pen.Color := clLime;
    Canvas.MoveTo(XPos, YMitte);
    repeat
      if FPos < FLenght then begin
         Seek(F, FPos);
         if not  EOF(F) then begin
            Read(F, B);
            YPos := B - 128; { LongInt }
            YPos := ((YPos * Height) div 256) + YMitte ;
            Canvas.LineTo(XPos, YPos);
         end;
      end;
      XPos := XPos + 1;
      FPos := FPos + Step;
    until FPos >= FLenght;
  finally
    CloseFile(F);
  end;

end;

procedure TDFWAV.PlayFileName(NewAction: Boolean);
var
  PFileName : Array [0..79] of Char;
begin
  MyPlay := NewAction;
  if MyPlay = True then
    begin
      StrPCopy(PFileName, MyFileName);
      sndPlaySound(PFileName,  wFlags[MyFlag]);
      MyPlay := False;
    end

end; {PlayFileName}

procedure TDFWAV.Beep;
{ erzeugt StandardBeep }
begin
  sndPlaySound('#',  1);                  { Fehlender Dateiname erzeugt Beep }
end; {Beep}

procedure TDFWAV.PlayWAV(FileName: String; Flags: Word);
{ spielt Filename mit Flags }
var
  PFileName : Array [0..79] of Char;
begin

  StrPCopy(PFileName, FileName);
  sndPlaySound(PFileName,  Flags);

end; {PlayWAV}

Procedure TDFWAV.WaitWAV;
var
  Res: Boolean;
begin
    Screen.Cursor := crHourglass;
    repeat
        Application.Processmessages;
        Res := sndPlaySound(nil, SND_SYNC Or SND_NODEFAULT Or SND_NOSTOP);
    Until Res = True;
    Screen.Cursor := crDefault;
end;

procedure Register;
begin
  RegisterComponents('DAF Controls', [TDFWAV]);
end;

end.
