unit Friends;

interface

uses Classes, SysUtils, Filters, StdCtrls;

type
  TFriend = class (TStreamable)
  public
    FMyFriend: TFriend;
    FName: String;
    constructor Create(const AName: string; AFriend: TFriend);
    constructor Load(Filter: TFilter); override;
    procedure ShowYourselfAndFriend(List: TStrings);
    procedure Store(Filter: TFilter); override;
  end;

  TFriends = class (TComponent)
  private
    FFriend: TFriend;
    FFriend1: TFriend;
    FFriend2: TFriend;
    FMemo: TMemo;
    FMemoryStream: TMemoryStream;
  protected
    procedure Load(Filter: TFilter);
    procedure Notification(AComponent: TComponent; Operation: TOperation); 
        override;
    procedure Store(Filter: TFIlter);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure FreeFriends;
    procedure LoadFriends;
    procedure MakeFriends;
    procedure ShowFriends;
    procedure StoreFriends;
  published
    property Memo: TMemo read FMemo write FMemo;
  end;


procedure Register;

implementation

uses MMUtils;

procedure Register;
begin
  RegisterComponents('MMDemo', [TFriends]);
end;

{
*********************************** TFriend ***********************************
}
constructor TFriend.Create(const AName: string; AFriend: TFriend);
begin
  inherited Create;
  FName := AName;
  FMyFriend := AFriend;
end;

constructor TFriend.Load(Filter: TFilter);
begin
  inherited Load(Filter);
  FMyFriend := Filter.Get as TFriend;
  FName := Filter.ReadStr;
end;

procedure TFriend.ShowYourselfAndFriend(List: TStrings);
begin
  List.Add(FName);
  if FMyFriend <> nil then FMyFriend.ShowYourSelfAndFriend(List);
end;

procedure TFriend.Store(Filter: TFilter);
begin
  inherited Store(Filter);
  Filter.Put(FMyFriend);
  Filter.WriteStr(FName);
end;

{
*********************************** TFriends ***********************************
}
constructor TFriends.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMemoryStream := TMemoryStream.Create;
end;

destructor TFriends.Destroy;
begin
  FreeFriends;
  FMemoryStream.Free;
  inherited Destroy;
end;

procedure TFriends.FreeFriends;
begin
  FFriend.Free;
  FFriend := nil;
  FFriend1.Free;
  FFriend1 := nil;
  FFriend2.Free;
  FFriend2 := nil;
end;

procedure TFriends.Load(Filter: TFilter);
begin
  { Workaround, this line is needed otherwise you get a ListIndex out of bounds. }
  FFriend := Filter.Get as TFriend;
  FFriend1 := Filter.Get as TFriend;
  FFriend2 := Filter.Get as TFriend;
end;

procedure TFriends.LoadFriends;
var
  Cyclic: TCycleFilter;
begin
  FreeFriends;
  FMemoryStream.Seek(0,0);
  Cyclic := TCycleFilter.Create( FMemoryStream);
  try
    Cyclic.OwnsStream := False;
    { A cycle filter will make sure that every object is loaded only once.
      This means you can load a maze of interconnected objects without
      fear of creating duplicate objects. }
    Load(Cyclic);
  except
    Cyclic.Free;
  end;
end;

procedure TFriends.MakeFriends;
begin
  FreeFriends;
  { Create a linked list of friends. }
  FFriend2 := TFriend.Create('Third Friend', nil);
  FFriend1 := TFriend.Create('Second Friend', FFriend2);
  FFriend := TFriend.Create('First Friend', FFriend1);
end;

procedure TFriends.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FMemo then FMemo := nil;
  end;
end;

procedure TFriends.ShowFriends;
begin
  if FMemo = nil then
    raise Exception.Create('Cannot show friends, no memo attached to TFriends component !')
  else
  begin
    FMemo.Lines.Clear;
    if FFriend <> nil then FFriend.ShowYourselfAndFriend(FMemo.Lines);
  end;
end;

procedure TFriends.Store(Filter: TFIlter);
begin
  { Workaround, this line is needed otherwise you get a ListIndex out of bounds. }
  Filter.Put(FFriend);
  Filter.Put(FFriend1);
  Filter.Put(FFriend2);
end;

procedure TFriends.StoreFriends;
var
  Cyclic: TCycleFilter;
begin
  FMemoryStream.Seek(0,0);
  Cyclic := TCycleFilter.Create( FMemoryStream);
  try
    { A cycle filter will make sure that every object is stored only once.
      This means you can store a maze of interconnected objects without
      fear of your program crashing because of cyclic object references. }
    Cyclic.OwnsStream := False;
    Store(Cyclic);
  except
    Cyclic.Free;
  end;
end;


initialization
  RegisterStreamable(TFriend);
end.
