unit MMutils;

interface

uses SysUtils, Classes;

function MaxnInt(const Values: array of LongInt): LongInt;
  { returns minimum of n longints }

function MinnInt(const Values: array of LongInt): LongInt;
  { returns maximum of n LongInts }

function Min2Int(X, Y: Integer): Integer;
  { returns minimum from x, y }

function Max2Int(X, Y: Integer): Integer;
  { Returns maximum from x, y }

function MinLong(X, Y: LongInt): LongInt;
  { returns minimum from x, y }

function MaxLong(X, Y: LongInt): LongInt;
  { Returns maximum from x, y }

function MinWord(X, Y: Word): Word;
  { returns minimum from x, y }

function MaxWord(X, Y: Word): Word;
  { Returns maximum from x, y }

function MinByte(X, Y: Byte): Byte;
  { returns minimum from x, y }

function MaxByte(X, Y: Byte): Byte;
  { Returns maximum from x, y }

function Max2(X, Y: Extended): Extended;
  { Returns maximum from x, y }

function Max3(X, Y, Z: Extended): Extended;
  { Returns maximum from x, y, z }

function Min2(X, Y: Extended): Extended;
  { returns minimum from x, y }

function Min3(X, Y, Z: Extended): Extended;
  { returns minimum from x, y, z }

function CompareInt(L1, L2: LongInt): Integer;
  { Returns 1 if L1 > L2, -1 if L1 < l2 and 0 if L1 = L2 }

procedure SetBits(var State: Word; ABits: Word; Enable: Boolean);
  { Enables ABits bits in State which is treated like a bit array }

function BitState(State, ABits: Word): Boolean;
  { Returns True if State and ABits  <> 0 , so if any bit is set }

function DelChar(const S: String; C: Char): String;
  { Removes any character C from S }

function DelLeadChar(const S: String; C: Char): String;
  { Removes any leading character C from S }

function DelTailChar(const S: String; C: Char): String;
  { Removes any trailing character C from S }

function DelLeadTailChar(const S: String; C: Char): String;
  { Removes any leading and trailing character C from S }

function DelBlanks(const S: string): string;
  { Removes any #32 from s }

function DelLeadBlanks(const s: string): string;
  { Removes leading spaces }

function DelTailBlanks(const s: string) : string;
  { Removes trailing spaces }

function DelLeadTailBlanks(const s: string) : string;
  { Removes leading spaces and trailing spaces }

function CharString(C: Char; Cnt: Byte): String;
  { Returns a string of length (Cnt), filled with C }

function BlankString(Cnt: Byte): String;
  { Returns a string a Cnt blanks }

function AbbrStr(const Source: String; MaxLen: Byte): String;
  { Returns a string with length <= MaxLen. Abbreviating on words }

function StringValue(S: PString): String;
  { Returns S^ or '' if s = nil }

function JustFileName(const FileName: String): string;
  { Splits Filename in DirStr + NameStr + ExtStr. Returns NameStr only }

function CopyFile(const SourceName, TargetName: String): Boolean;
  { Copies Source filename to Target file name, returns true if OK }

function CommandPath: String;
  { full Returns executable path }

procedure Delay(MSec: Word);
  { Delays MSec }

procedure WriteHistory(const FileName: String; History: TStrings; Number: Integer);

procedure ReadHistory(const FileName: String; History: TStrings; Number: Integer);


implementation

uses WinTypes, WinProcs, IniFiles;

{
******************** simple type procedures and functions *******************
}
function MaxnInt(const Values: array of LongInt): LongInt;
var I: Integer;
begin
  Result := Values[Low(Values)];
  for I := Low(Values) + 1 to High(Values) do
    if Values[I] > Result then Result := Values[I];
end;

function MinnInt(const Values: array of LongInt): LongInt;
var I: Integer;
begin
  Result := Values[Low(Values)];
  for I := Low(Values) + 1 to High(Values) do
    if Values[I] < Result then Result := Values[I];
end;

function Min2Int(X, Y: Integer): Integer; assembler;
asm
        MOV     AX,X
        CMP     AX,Y
        JLE     @@1
        MOV     AX,Y
@@1:
end;

function Max2Int(X, Y: Integer): Integer; assembler;
asm
        MOV     AX,X
        CMP     AX,Y
        JGE     @@1
        MOV     AX,Y
@@1:
end;

function MaxLong(x, y: LongInt): LongInt;
begin
  if x > y then
    Result := x
  else
    Result := y;
end;

function MinLong(x, y: LongInt): LongInt;
begin
  if x < y then
    Result := x
  else
    Result := y;
end;

function MinWord(X, Y: Word): Word; assembler;
asm
        MOV     AX,X
        CMP     AX,Y
        JBE     @@1
        MOV     AX,Y
@@1:
end;

function MaxWord(X, Y: Word): Word; assembler;
asm
        MOV     AX,X
        CMP     AX,Y
        JAE     @@1
        MOV     AX,Y
@@1:
end;

function MinByte(X, Y: Byte): Byte;
begin
  if X > Y then
    Result := Y
  else
    Result := X;
end;

function MaxByte(X, Y: Byte): Byte;
begin
  if X < Y then
    Result := Y
  else
    Result := X;
end;

function Max2(X, Y: Extended): Extended;
begin
  if X > Y then Result := X else Result := Y;
end;

function Max3(X, Y, Z: Extended): Extended;
begin
  Result := Max2(Max2(X, Y), Z);
end;

function Min2(X, Y: Extended): Extended;
begin
  if X < Y then Result := X else Result := Y;
end;

function Min3(X, Y, Z: Extended): Extended;
begin
  Min3 := Min2(Min2(X, Y), Z);
end;

function CompareInt(L1, L2: LongInt): Integer;
begin
  if L1 < L2 then Result := -1
  else
    if L1 > L2 then Result := 1
    else  Result := 0;
end;

procedure SetBits(var State: Word; ABits: Word; Enable: Boolean);
begin
  if Enable then
    State := State or ABits
  else
    State := State and not ABits;
end;

function BitState(State, ABits: Word): Boolean;
begin
  Result := (State and ABits <> 0);
end;

function DelChar(const S: String; C: Char): String;
var Temp: String;
    I: integer;
begin
  Temp := '';
  for I := 1 to Length(S) do
    if S[I] <> C then
      Temp := Temp + S[I];
  Result := Temp;
end;

function DelLeadChar(const S: String; C: Char): String;
begin
  Result := S;
  while Pos(C, Result) = 1 do
    Delete(Result, 1, 1);
end;

function DelTailChar(const S: String; C: Char): String;
begin
  Result := S;
  if Length(Result) > 0 then
    while Result[Length(Result)] = C do
      Dec(Result[0]);
end;

function DelLeadTailChar(const S: String; C: Char) : String;
begin
  Result := DelTailChar(DelLeadChar(S, C), C);
end;

function DelBlanks(const S: String): String;
begin
  Result := DelChar(S, ' ');
end;

function DelLeadBlanks(const S: String): String;
begin
  Result := DelLeadChar(S, ' ');
end;

function DelTailBlanks(const S: String): String;
begin
  Result := DelTailChar(S, ' ');
end;

function DelLeadTailBlanks(const S: String): String;
begin
  Result := DelLeadTailChar(S, ' ');
end;

function CharString(C: Char; Cnt: Byte): String;
begin
  FillChar(Result, SizeOf(Result), C);
  Result[0] := Chr(Cnt);
end;

function BlankString(Cnt: Byte): String;
begin
  Result := CharString(#32, Cnt);
end;

function AbbrStr(const Source: String; MaxLen: Byte): String;
begin
  if Length(Source) > MaxLen then
  begin
    Result := Copy(Source, 1, MaxLen);
    if (Source[MaxLen] <> ' ') and (Source[MaxLen + 1] <> ' ') then
    if MaxLen > 1 then
    begin
      Result[MaxLen] := '.';
      Result[MaxLen - 1] := '.';
    end;
  end
  else
    Result := Source;
end;

function StringValue(S: PString): String;
begin
  if Assigned(S) then
    Result := S^
  else
    Result := '';
end;

function JustFileName(const FileName: String): String;
var I: Integer;
begin
  Result := ExtractFileName(FileName);
  I := Pos('.', Result);
  if I > 0 then
    Result := Copy(Result, 1, I - 1)
end;

function CommandPath: String;
var Path: TFileName;
begin
  Path := ParamStr(0);
  Result := ExtractFilePath(Path);
end;

function CopyFile(const SourceName, TargetName: String): Boolean;
const sizeBlock = 1024;
var Source, Target: TFileStream;
    Buf: array[0..Pred(sizeBlock)] of Byte;
    M: Word;
    Rest: LongInt;
begin
  Result := False;
  if FileExists(SourceName) and not FileExists(TargetName) then
  begin
    Source := TFileStream.Create(SourceName, fmOpenRead);
    try
      Target := TFileStream.Create(TargetName, fmCreate);
      try
        Rest := Source.Size;
        while Rest > 0 do
        begin
          if Rest > sizeBlock then M := sizeBlock else M := Rest;
          Source.Read(Buf, M);
          Target.Write(Buf, M);
          Dec(Rest, M);
        end;
        Result := True;
      finally
        Target.Destroy;
      end;
    finally
      Source.Destroy;
    end;
  end;
end;

procedure Delay(MSec: Word);
var
  Time: LongInt;
  Msg: TMsg;
begin
  Time := GetTickCount;
  Inc(Time, MSec);
  while GetTickCount < Time do
    if PeekMessage(Msg, 0, 0, 0, PM_REMOVE) then
    begin
      TranslateMessage(Msg);
      DispatchMessage(Msg)
    end;
end;

procedure HandleMessages(Count: Word);
var Msg: TMsg;
begin
  while (Count > 0) and PeekMessage(Msg, 0, 0, 0, PM_REMOVE) do
  begin
    TranslateMessage(Msg);
    DispatchMessage(Msg);
    Dec(Count);
  end;
end;

const SectionTemplate = 'History_';
      KeyTemplate     = 'H';
      stCount         = 'Count';

procedure WriteHistory(const FileName: String; History: TStrings; Number: Integer);
var IniFile: TIniFile;
    Section: String[13];
    I: Integer;
begin
  Section := SectionTemplate + IntToStr(Number);
  IniFile := TIniFile.Create(FileName);
  try
    IniFile.EraseSection(Section);
    IniFile.WriteInteger(Section, stCount, History.Count);
    for I := 0 to History.Count - 1 do
      IniFile.WriteString(Section, KeyTemplate + IntToStr(I), History[I]);
  finally
    IniFile.Free;
  end;
end;

procedure ReadHistory(const FileName: String; History: TStrings; Number: Integer);
var IniFile: TIniFile;
    Section: String[13];
    S: String;
    Cnt, I: Integer;
begin
  History.Clear;
  Section := SectionTemplate + IntToStr(Number);
  IniFile := TIniFile.Create(FileName);
  try
    Cnt := IniFile.ReadInteger(Section, stCount, 0);
    for I := 0 to Cnt - 1 do
    begin
      S := IniFile.ReadString(Section, KeyTemplate + IntToStr(I), '');
      History.Add(S)
    end;
  finally
    IniFile.Free;
  end;
end;


end.
